#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/colour_table/colour_table.rb'
# =========================================================================== #
module Colours

  require 'colours/toplevel_methods/use_colours.rb'
  require 'colours/html_colours/html_colours.rb'

  # ========================================================================= #
  # === HASH_NEW_COLOURS
  #
  # This is the new hash, which we can transition-into when necessary.
  #
  # It is only rarely used these days, though (as of the year 2018).
  #
  # If you want to use these colours, you can use this simpler API:
  #
  #    Colourss.set_method_to_colour_table :new_colours
  #
  # ========================================================================= #
  HASH_NEW_COLOURS = {
    default_colour: :green,        
    sargument:      :bold_brown,    # === sarg()
    scomments:      :bold_blue,     # === scomments()
    sdir:           :bold_green,    # === sdir()
    sfancy:         :bold_yellow,   # === sfancy()
    sfile:          :brown,         # === sfile()
    simportant:     :pink,          # === simportant()
    ssymlink:       :teal,          # === ssymlink()
    swarn:          :bold_red,      # === swarn
    snormal:        :light_grey     # === snormal()
  }

  # ========================================================================= #
  # === Colours.update_the_colour_codes_with                     (update tag)
  # ========================================================================= #
  def self.update_the_colour_codes_with(i = HASH_NEW_COLOURS)
    @colour_table.update(i)
  end; self.instance_eval { alias update_colour_table_with update_the_colour_codes_with } # === Colours.update_colour_table_with
       self.instance_eval { alias update_the_colour_codes  update_the_colour_codes_with } # === Colours.update_the_colour_codes
       self.instance_eval { alias update_colour_codes      update_the_colour_codes_with } # === Colours.update_colour_codes

  # ========================================================================= #
  # === @colour_table                      (default tag, default colours tag)
  #
  # This toplevel instance variable has the default colours assigned to
  # the different methods. If you ever wish to change the default colours
  # in use, you can do so by manipulating this instance variable.
  #
  # Keep it sorted alphabetically.
  # ========================================================================= #
  @colour_table = {
    default_colour:  :green,
    sfancy:          :bold_green,   # === sfancy()
    sargument:       :bold_green,   # === sargument()
    scomments:       :grey,         # === scomments()
    sdir:            :chocolate,    # === sdir()
    sfile:           :saddlebrown,  # === sfile()
    simportant:      :gold,         # === simportant()
    ssymlink:        :lightblue,    # === ssymlink()
    swarn:           :bold_red,     # === swarn()
    snormal:         :light_grey    # === snormal()
  }

  if @use_html_colours and File.exist?(FILE_USE_THESE_VALUES_FOR_THE_COLOUR_METHODS)
    these_colours = YAML.load_file(FILE_USE_THESE_VALUES_FOR_THE_COLOUR_METHODS).
                    transform_keys(&:to_sym)
    update_colour_table_with(these_colours)
  end

  # ========================================================================= #
  # === Colours.map_this_symbol_to_that_html_colour
  #
  # The job of this method is to map entries such as :bold_green to
  # a corresponding HTML colour.
  # ========================================================================= #
  def self.map_this_symbol_to_that_html_colour(i)
    if is_a_html_colour? i
      i
    else
      case i
      # === :cyanb
      when :cyanb
        :lightseagreen
      # === :light_grey
      when :light_grey
        :lightgray
      # === :bold_blue
      when :bold_blue
        :slateblue
      when :bold_red, :redb
        :crimson
      when :bold_yellow
        :goldenrod
      when :bold_green
        :seagreen
      else
        puts 'Unregistered input into the method '+__method__.to_s+': '+i.to_s
      end
    end
  end

  # ========================================================================= #
  # === Colours.map_this_symbol_to_that_256_colour
  #
  # This method will take input such as :bold_green and associate it with
  # a corresponding 256-colour. Thus, the resulting Symbol that is
  # found here should correspond to a method on module SupportFor256Colours.
  # ========================================================================= #
  def self.map_this_symbol_to_that_256_colour(i)
    case i
    when :boldgreen,
         :bold_green
      :chartreuse2
    when :teal
      :teal
    when :bold_yellow
      :yellow2
    when :gold
      :gold3
    when :bold_blue
      :lightslateblue
    when :bold_brown
      :darkred
    when :brown
      :maroon
    when :pink
      :hotpink3
    when :lightgrey,
         :light_grey
      :grey66
    when :bold_red
      :red1
    when :lightblue,
         :light_blue
      :deepskyblue3
    when :grey
      :grey53
    when :green
      :chartreuse4
    else
      puts "Not known input to method #{__method__.to_s}: #{i}"
    end
  end

  # ========================================================================= #
  # === use_new_colour_codes
  #
  # Use this method to assign new colour codes.
  # We assume that you must pass a hash to this method.
  #
  # Usage example:
  #
  #   new_hash = {
  #     :sfancy         => 'green',
  #     :simportant     => 'teal',
  #     :default_colour => 'grey',
  #     :sfile          => 'magenta',
  #     :sdir           => 'cyan'
  #   }
  #   Colours.assign_new_colour_codes(new_hash)
  #
  # ========================================================================= #
  def use_new_colour_codes(
      i = HASH_NEW_COLOURS
    )
    @colour_table.update(i)
  end; alias assign_new_colour_codes use_new_colour_codes # === assign_new_colour_codes

  # ========================================================================= #
  # === Colours.set_colour_table
  #
  # This method can be used to modify the instance variable
  # @colour_table.
  #
  # The input to this method should be a Hash. It can also be a Symbol,
  # in which case we will try to find a Hash that corresponds to that
  # Symbol.
  #
  # If in doubt, pass in a Hash though.
  # ========================================================================= #
  def self.set_colour_table(
      i = :new_colours
    )
    case i
    when :new_colours,
         :new_colour_table,
         :default
      i = HASH_NEW_COLOURS
    end
    # ======================================================================= #
    # Note that we must ensure that all the necessary keys are there
    # in the given input. If this is not the case, then we will
    # determine which entries are missing, and assign them from the
    # instance variable @colour_table.
    # ======================================================================= #
    missing_entries = (
      i.keys - @colour_table.keys
    ).flatten
    unless missing_entries.empty?
      # ===================================================================== #
      # Push them onto i in this case.
      # ===================================================================== #
      missing_entries.each {|this_key|
        if i.has_key? this_key
          # All fine, pass through in this case.
        else
          add_this = missing_entries[this_key]
          i[key] = add_this

        end
      }
    end
    @colour_table = i
  end; self.instance_eval { alias assign_new_colour_codes    set_colour_table } # === Colours.assign_new_colour_codes
       self.instance_eval { alias set_method_to_colour_table set_colour_table } # === Colours.set_method_to_colour_table

  # ========================================================================= #
  # === Colours.use_new_colour_table
  # ========================================================================= #
  def self.use_new_colour_table
    set_colour_table(:new_colour_table)
  end

  # ========================================================================= #
  # === Colours.show_the_colour_table
  #
  # This will simply pretty-print the current colour table.
  # ========================================================================= #
  def self.show_the_colour_table
    pp @colour_table
  end

  # ========================================================================= #
  # === Colours.shuffle
  #
  # Shuffle the above table. The available colours to take here will
  # be from the HtmlColours namespace.
  # ========================================================================= #
  def self.shuffle
    # ======================================================================= #
    # Point to the html "table" to use.
    # ======================================================================= #
    _ = ::Colours::HtmlColours.all?.shuffle
    @colour_table = {
      simp:      _.shift,
      sdir:      _.shift,
      sfile:     _.shift,
      swarn:     _.shift,
      sfancy:    _.shift,
      sargument: _.shift,
      ssymlink:  _.shift
    }
  end

  # ========================================================================= #
  # === Colours.colour_table?
  #
  # This method will feedback the available colour-table.
  #
  # To invoke this method, try the following code:
  #
  #   pp Colours.colours?
  #
  # ========================================================================= #
  def self.colour_table?
    @colour_table
  end; self.instance_eval { alias method_to_colour_table                  colour_table? } # === Colours.method_to_colour_table
       self.instance_eval { alias method_to_colour_table?                 colour_table? } # === Colours.method_to_colour_table?
       self.instance_eval { alias colours?                                colour_table? } # === Colours.colours?
       self.instance_eval { alias table?                                  colour_table? } # === Colours.table?
       self.instance_eval { alias table_colour_methods_to_specific_colour colour_table? } # === Colours.table_colour_methods_to_specific_colour
       self.instance_eval { alias colour_methods_to_specific_colour?      colour_table? } # === Colours.colour_methods_to_specific_colour?

end

if __FILE__ == $PROGRAM_NAME
  pp Colours::HASH_NEW_COLOURS  
end