#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal =  true
# =========================================================================== #
# require 'colours/constants/hash_simple_colours.rb'
# =========================================================================== #
module Colours

  # ========================================================================= #
  # === Colours::HASH_SIMPLE_COLOURS
  #
  # This hash will only store one name for the corresponding colour code.
  # ========================================================================= #
  HASH_SIMPLE_COLOURS = {
    black:         '0;30',
    red:           '0;31',
    green:         '0;32',
    yellow:        '0;33',
    blue:          '0;34',
    magenta:       '0;35',
    cyan:          '0;36',
    light_gray:    '0;37',
    # ======================================================================= #
    # Next, the secondary-colours:
    # ======================================================================= #
    dark_gray:     '1;30',
    light_red:     '1;31',
    light_green:   '1;32',
    brown:         '1;33',
    light_blue:    '1;34',
    light_magenta: '1;35',
    light_cyan:    '1;36',
    white:         '1;37'
  }

  # ========================================================================= #
  # === Colours::MAIN_COLOURS
  #
  # The main colours come next.
  #
  # Eight main colours are currently defined, according to:
  #
  #   http://en.wikipedia.org/wiki/ANSI_escape_code#Colors
  #
  # The range is:
  #
  #   Intensity:     0    1    2      3     4      5      6    7
  #   Normal:     Black  Red Green Yellow  Blue  Magenta Cyan White
  #
  # Yellow, in my opinion, looks more like brown.
  # ========================================================================= #
  MAIN_COLOURS = %i(
    black red green yellow blue magenta teal white
  ) # ^^^ An Array of Symbols.

  # ========================================================================= #
  # === Colours.available_main_colours?
  #
  # Feedback which main colours are available.
  # ========================================================================= #
  def self.available_main_colours?
    MAIN_COLOURS
  end; self.instance_eval { alias available_basic_colours? available_main_colours? } # === Colours.available_basic_colours?

  # ========================================================================= #
  # === main_colours
  #
  # This will return an Array with the symbols such as
  # :black, :red etc.. 
  # ========================================================================= #
  def main_colours
    ::Colours.available_main_colours?
  end

  # ========================================================================= #
  # === Colours::SECONDARY_COLOURS
  #
  # The secondary Colours.
  #
  # Colours which are not part of the MAIN_COLOURS Array.
  #
  # The secondary colours are:
  #
  #   Bright Black
  #   Bright Red
  #   Bright Green
  #   Bright Yellow
  #   Bright Blue
  #   Bright Magenta
  #   Bright Cyan
  #   Bright White
  #
  # ========================================================================= #
  SECONDARY_COLOURS = [
    :bright_black,
    :bright_red,
    :bright_green,
    :bright_yellow,
    :bright_blue,
    :bright_magenta,
    :bright_cyan,
    :bright_white
  ]

  # ========================================================================= #
  # === ALL_COLOURS
  #
  # Simply combine the two Arrays, MAIN_COLOURS and SECONDARY_COLOURS,
  # into one aggregate Array.
  # ========================================================================= #
  ALL_COLOURS = MAIN_COLOURS +
                SECONDARY_COLOURS

  # ========================================================================= #
  # === Colours.really_all_colours
  #
  # This method will return an Array that may look like this:
  #
  #   [:black, :red, :green, :yellow, :blue, :magenta, :teal,
  #    :white, :bright_black, :bright_red, :bright_green,
  #    :bright_yellow, :bright_blue, :bright_magenta,
  #    :bright_cyan, :bright_white]
  #
  # ========================================================================= #
  def self.really_all_colours
    ALL_COLOURS # This is the conjoint Array.
  end; self.instance_eval { alias all_available_colours really_all_colours } # === Colours.all_available_colours

  # ========================================================================= #
  # === Colours.is_a_valid_colour?
  #
  # This method allows you to check if a given input-string is a
  # valid colour.
  #
  # A "valid" colour is one that the colours project supports.
  #
  # For example, the string "lightblue" is a valid colour.
  #
  # Usage example:
  #
  #   Colours.is_a_valid_colour? 'lightblue'
  #
  # ========================================================================= #
  def self.is_a_valid_colour?(i)
    ALL_COLOURS.include? i.to_sym
  end

end