#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal =  true
# =========================================================================== #
# This file primarily stores constants, including AnsiColours-related
# escape sequences. Additionally it will also store methods can refer
# or make use of these constants, such as Colours.teal(), and so forth.
#
# The different escape sequences can be combined into a single escape
# sequence by separating them via semicolons. For example, to create
# a section of text that is bold text with a yellow foreground and
# blue background, the escape sequence would be:
#
#   ^[[1;33;44m
#
# Keep in mind that in bash the ESC code can be either of the following:
#
#    \e
#    \033 (octal)
#    \x1B (hexadecimal)
#
# BOLD colour constants will have a "1" as part of their escape sequence.
# Note that the bold colours are sometimes also called the "bright colours".
# =========================================================================== #
# To see which colours are supported/supportable, for each terminal,
# have a look at this link:
#
#   https://misc.flogisoft.com/bash/tip_colors_and_formatting#terminals_compatibility
#
# This wikipedia link is also somewhat useful for ANSI escape sequences
# in general:
#
#   http://en.wikipedia.org/wiki/ANSI_escape_code#Colors
#
# =========================================================================== #
# This file is tested in the file
# colours/test/testing_the_basic_colours_of_the_colours_project.rb
# =========================================================================== #
# The following table shows the colour-codes for the terminal:
#
#   Black       0;30   Dark Gray     1;30
#   Red         0;31   Light Red     1;31
#   Green       0;32   Light Green   1;32
#   Yellow      0;33   Brown         1;33
#   Blue        0;34   Light Blue    1;34
#   Magenta     0;35   Light Magenta 1;35
#   Cyan        0;36   Light Cyan    1;36
#   Light Gray  0;37   White         1;37
#
# These are also stored in the Hash called HASH_SIMPLE_COLOURS.
#
# =========================================================================== #
# require 'colours/constants/misc.rb'
# =========================================================================== #
module Colours

  require 'colours/constants/escape.rb'
  require 'colours/constants/hash_simple_colours.rb'
  require 'colours/constants/newline.rb'
  require 'colours/toplevel_methods/revert.rb'

  # ========================================================================= #
  # === REGEX_REMOVE_ANSI_ESCAPE_CHARACTERS
  #
  # This regex can be used to get rid of (some) AnsiEscape characters.
  # ========================================================================= #
  REGEX_REMOVE_ANSI_ESCAPE_CHARACTERS =
    /\033\[.*?m/
    REMOVE_ANSI_ESCAPE_CHARACTERS = REGEX_REMOVE_ANSI_ESCAPE_CHARACTERS # === REMOVE_ANSI_ESCAPE_CHARACTERS

  # ========================================================================= #
  # === REDRAW_THE_PROMPT
  #
  # The following constant is necessary in order for the background colour
  # to apply to the whole line and in order for the foreground colour to
  # apply to the cursor.
  # ========================================================================= #
  REDRAW_THE_PROMPT = "\e[K"

  # ========================================================================= #
  # === HOME_DIRECTORY_OF_USER_X
  #
  # This constant is only useful on my home system.
  # ========================================================================= #
  HOME_DIRECTORY_OF_USER_X = '/home/x/'

  # ========================================================================= #
  # === Colours::DEFAULT_TEXT
  # ========================================================================= #
  DEFAULT_TEXT = 'Hello World!'

  # ========================================================================= #
  # === Colours::IS_A_TEST
  # ========================================================================= #
  IS_A_TEST = ' This is just a simple colour test.' 

  # ========================================================================= #
  # === Colours::TEST_FILE
  # ========================================================================= #
  TEST_FILE =
    "#{HOME_DIRECTORY_OF_USER_X}programming/ruby/src/colours/test/testing_konsole_submodule.rb"

  # ========================================================================= #
  # === LEFT
  #
  # Leading on the left side, for RGB value setting. This is the beginning
  # of a special instruction. Note that "\e[38" should be equal to "\x1b",
  # but IRB reports that this is not the case so.
  #
  # Examples on the www show usage such as this:
  #
  #   printf "\x1b[${bg};2;${red};${green};${blue}m\n"
  #   printf "\x1b[38;2;255;100;0mTRUECOLOR\x1b[0m\n"
  # 
  # ========================================================================= #
  LEFT = "\e[38;2;".dup

  # ========================================================================= #
  # === Colours.left?
  # ========================================================================= #
  def self.left?
    LEFT
  end

  # ========================================================================= #
  # === left?
  # ========================================================================= #
  def left?
    Colours.left?
  end

  # ========================================================================= #
  # === COLOUR_BLACK
  # ========================================================================= #
  COLOUR_BLACK        = "#{ESC}0;30m"
    COLOUR_GRAY       = COLOUR_BLACK # === COLOUR_GRAY
    BLACK             = COLOUR_BLACK # === BLACK

  # ========================================================================= #
  # === COLOUR_LIGHT_CYAN
  # ========================================================================= #
  COLOUR_LIGHT_CYAN   = "#{ESC}1;36m"
    BOLD_CYAN         = COLOUR_LIGHT_CYAN # === BOLD_CYAN
    COLOUR_BOLD_TEAL  = COLOUR_LIGHT_CYAN # === COLOUR_BOLD_TEAL
    BRIGHT_CYAN       = COLOUR_LIGHT_CYAN # === BRIGHT_CYAN
    BRIGHT_TEAL       = COLOUR_LIGHT_CYAN # === BRIGHT_TEAL
    CYANB             = COLOUR_LIGHT_CYAN # === CYANB
    BOLD_TEAL         = COLOUR_LIGHT_CYAN # === BOLD_TEAL

  # ========================================================================= #
  # === COLOUR_CYAN
  #
  # This is also called TEAL, sort of.
  # ========================================================================= #
  COLOUR_CYAN         = "#{ESC}0;36m"
    TEAL              = COLOUR_CYAN # === TEAL
    CYAN              = COLOUR_CYAN # === CYAN
    COLOUR_TEAL       = COLOUR_CYAN # === COLOUR_TEAL

  # ========================================================================= #
  # === COLOUR_BROWN
  # ========================================================================= #
  COLOUR_BROWN         = "#{ESC}1;33m"
    BOLD_YELLOW        = COLOUR_BROWN # === BOLD_YELLOW
    GOLD               = COLOUR_BROWN # === GOLD
    BRIGHT_BROWN       = COLOUR_BROWN # === BRIGHT_BROWN
    BRIGHT_YELLOW      = COLOUR_BROWN # === BRIGHT_YELLOW
    BOLDYELLOW         = COLOUR_BROWN # === BOLDYELLOW
    BYEL               = COLOUR_BROWN # === BYEL
    BYELLOW            = COLOUR_BROWN # === BYELLOW
    COLOUR_BOLD_YELLOW = COLOUR_BROWN # === COLOUR_BOLD_YELLOW

  # ========================================================================= #
  # === COLOUR_LIGHT_GRAY
  #
  # This is also known as white.
  # ========================================================================= #
  COLOUR_LIGHT_GRAY   = "#{ESC}0;37m"
    WHITE             = COLOUR_LIGHT_GRAY # === WHITE
    COLOUR_LIGHT_GREY = COLOUR_LIGHT_GRAY # === COLOUR_LIGHT_GREY

  # ========================================================================= #
  # === COLOUR_WHITE
  #
  # This is bold white, actually.
  # ========================================================================= #
  COLOUR_WHITE        = "#{ESC}1;37m"
    BOLD_WHITE        = COLOUR_WHITE # === BOLD_WHITE 
    BRIGHT_WHITE      = COLOUR_WHITE # === BRIGHT_WHITE
    BWHITE            = COLOUR_WHITE # === BWHITE
    COLOUR_BOLD_WHITE = COLOUR_WHITE # === COLOUR_BOLD_WHITE

  # ========================================================================= #
  # === COLOUR_DARK_GRAY
  #
  # This is also called "black bold".
  # ========================================================================= #
  COLOUR_DARK_GRAY   = "#{ESC}1;30m"
    GREY             = COLOUR_DARK_GRAY # === GREY
    GRAY             = COLOUR_DARK_GRAY # === GRAY
    CGREY            = COLOUR_DARK_GRAY # === CGREY
    BRIGHT_BLACK     = COLOUR_DARK_GRAY # === BRIGHT_BLACK
    BOLD_BLACK       = COLOUR_DARK_GRAY # === BOLD_BLACK

  # ========================================================================= #
  # == COLOUR_RED                                                   (red tag)
  # ========================================================================= #
  COLOUR_RED          = "#{ESC}0;31m" # This is thin, darkred.
    RED  = COLOUR_RED # === RED
    CRED = COLOUR_RED # === CRED

  # ========================================================================= #
  # === COLOUR_GREEN
  # ========================================================================= #
  COLOUR_GREEN        = "#{ESC}0;32m" # This is lightgreen.
    GREEN     = COLOUR_GREEN # === GREEN
    CGREEN    = COLOUR_GREEN # === CGREEN
    DARKGREEN = COLOUR_GREEN # === DARKGREEN

  # ========================================================================= #
  # === COLOUR_BLUE
  # ========================================================================= #
  COLOUR_BLUE         = "#{ESC}0;34m"
    BLUE = COLOUR_BLUE # === BLUE

  # ========================================================================= #
  # === COLOUR_LIGHT_GREEN
  # ========================================================================= #
  COLOUR_LIGHT_GREEN  = "#{ESC}1;32m"
    BOLD_GREEN        = COLOUR_LIGHT_GREEN # === BOLD_GREEN
    COLOUR_BOLD_GREEN = COLOUR_LIGHT_GREEN # === COLOUR_BOLD_GREEN
    BRIGHT_GREEN      = COLOUR_LIGHT_GREEN # === BRIGHT_GREEN
    LIGHTGREEN        = COLOUR_LIGHT_GREEN # === LIGHTGREEN
    BGREEN            = COLOUR_LIGHT_GREEN # === BGREEN

  # ========================================================================= #
  # === COLOUR_MAGENTA
  # ========================================================================= #
  COLOUR_MAGENTA    = "#{ESC}0;35m"
    PURPLE        = COLOUR_MAGENTA # === PURPLE
    COLOUR_PURPLE = COLOUR_MAGENTA # === COLOUR_PURPLE
    MAGENTA       = COLOUR_MAGENTA # === MAGENTA

  # ========================================================================= #
  # === COLOUR_LIGHT_BLUE
  #
  # This is also known as "bold blue".
  # ========================================================================= #
  COLOUR_LIGHT_BLUE   = "#{ESC}1;34m"
    BOLD_BLUE         = COLOUR_LIGHT_BLUE # === BOLD_BLUE
    COLOUR_BOLD_BLUE  = COLOUR_LIGHT_BLUE # === COLOUR_BOLD_BLUE 
    BRIGHT_BLUE       = COLOUR_LIGHT_BLUE # === BRIGHT_BLUE
    LIGHT_BLUE        = COLOUR_LIGHT_BLUE # === LIGHT_BLUE
    LIGHTBLUE         = COLOUR_LIGHT_BLUE # === LIGHTBLUE
    BBLUE             = COLOUR_LIGHT_BLUE # === BBLUE
    BOLDBLUE          = COLOUR_LIGHT_BLUE # === BOLDBLUE

  # ========================================================================= #
  # === Colours::BLACK_ON_WHITE_BACKGROUND
  #
  # Usage example:
  #
  #   puts Colours::BLACK_ON_WHITE_BACKGROUND+'Hello world!'+Colours::RESET
  #
  # ========================================================================= #
  BLACK_ON_WHITE_BACKGROUND = "#{ESC}0;30;47m"
    BLACK_ON_WHITE_BG = BLACK_ON_WHITE_BACKGROUND # === BLACK_ON_WHITE_BG

  # ========================================================================= #
  # === COLOUR_YELLOW
  # ========================================================================= #
  COLOUR_YELLOW       = "#{ESC}0;33m"
    BROWN             = COLOUR_YELLOW # === BROWN
    YEL               = COLOUR_YELLOW # === YEL
    YELLOW            = COLOUR_YELLOW # === YELLOW
    BOLD_BROWN        = COLOUR_YELLOW # === BOLD_BROWN

  # ========================================================================= #
  # === COLOUR_LIGHT_MAGENTA
  #
  # This is not the "normal" magenta, but the light variant; also
  # called light purple.
  # ========================================================================= #
  COLOUR_LIGHT_MAGENTA = "#{ESC}1;35m"
    LIGHT_PURPLE        = COLOUR_LIGHT_MAGENTA # === LIGHT_PURPLE
    COLOUR_LIGHT_PURPLE = COLOUR_LIGHT_MAGENTA # === COLOUR_LIGHT_PURPLE
    PINK                = COLOUR_LIGHT_MAGENTA # === PINK
    COLOUR_PINK         = COLOUR_LIGHT_MAGENTA # === COLOUR_PINK
    BRIGHT_PINK         = COLOUR_LIGHT_MAGENTA # === BRIGHT_PINK
    BRIGHT_MAGENTA      = COLOUR_LIGHT_MAGENTA # === BRIGHT_MAGENTA

  # ========================================================================= #
  # === COLOUR_LIGHT_RED
  # ========================================================================= #
  COLOUR_LIGHT_RED    = "#{ESC}1;31m"
    LIGHT_RED         = COLOUR_LIGHT_RED # === LIGHT_RED
    BOLD_RED          = COLOUR_LIGHT_RED # === BOLD_RED
    COLOUR_BOLD_RED   = COLOUR_LIGHT_RED # === COLOUR_BOLD_RED 
    BRIGHT_RED        = COLOUR_LIGHT_RED # === BRIGHT_RED
    BOLDRED           = COLOUR_LIGHT_RED # === BOLDRED
    REDB              = COLOUR_LIGHT_RED # === REDB
    BRED              = COLOUR_LIGHT_RED # === BRED
  # ========================================================================= #
  # === ARRAY_REGISTERED_ANSI_COLOURS
  #
  # This array exists mostly for testing-purposes. 
  # ========================================================================= #
  ARRAY_REGISTERED_ANSI_COLOURS = [

    COLOUR_BLACK,
    COLOUR_RED,
    COLOUR_GREEN,
    COLOUR_YELLOW,
    COLOUR_BLUE,
    COLOUR_MAGENTA,
    COLOUR_TEAL,
    COLOUR_LIGHT_GREY,

    COLOUR_BOLD_RED,
    COLOUR_BOLD_GREEN,
    COLOUR_BOLD_YELLOW,
    COLOUR_BOLD_BLUE,
    COLOUR_PINK,
    COLOUR_BOLD_TEAL,
    COLOUR_BOLD_WHITE,

    COLOUR_RESET

  ]

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  include Colours
  e CYAN+'Hi there.'+RESET
  e BWHITE+'Hi there.'+BBLUE+' Yo there.'+RESET
  e WHITE+'Hi there.'+BBLUE+' Yo there.'+RESET
  e BLACK_ON_WHITE_BACKGROUND+'Hello world!'+RESET
  e BRIGHT_RED+'Hi there.'+BBLUE+' Yo there.'+RESET
  e BRIGHT_BLUE+'Hi there.'+BWHITE+' Yo there.'+RESET
  e
  e 'And testing individual colours for the characters next:'
  e
  e BLACK+' A '+
    RED+' B '+
    GREEN+' C '+
    BROWN+' D '+
    BLUE+' E '+
    MAGENTA+' F '+
    TEAL+' G '+
    WHITE+' H '+
    RESET
  e
  e 'Same as above but with the bright colours instead:'
  e
  e BRIGHT_BLACK+' A '+BRIGHT_RED+' B '+BRIGHT_GREEN+' C '+
    BRIGHT_BROWN+' D '+BRIGHT_BLUE+' E '+BRIGHT_MAGENTA+
    ' F '+BRIGHT_TEAL+' G '+BRIGHT_WHITE+' H '+RESET
  e
end