#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/toplevel_methods/misc.rb'
# =========================================================================== #
module Colours

  require 'colours/html_colours/html_colours.rb'
  require 'colours/toplevel_methods/esystem.rb'
  require 'colours/toplevel_methods/open_this_file.rb'
  require 'colours/toplevel_methods/remove_escape_sequence.rb'

  # ========================================================================= #
  # === Colours.colour_method_or_display_the_content_of_the_file_or_use_via_pipe
  #
  # This method has an awful name, but its core use case is simple.
  #
  # It will handle commandline instructions such as:
  #
  #   orange Hey there
  #   slateblue how are you doing?
  #
  # Usage example from within ruby:
  #
  #   Colours.testing123('abc', :orange)
  #
  # ========================================================================= #
  def self.colour_method_or_display_the_content_of_the_file_or_use_via_pipe(
      i                  = ARGF,
      name_of_the_colour = $PROGRAM_NAME
    )
    require 'colours/autogenerated/support_for_html_colours.rb'
    # ======================================================================= #
    # === :read
    # ======================================================================= #
    if i.respond_to? :read
      i = i.read
    elsif i and File.exist?(i)
      i = File.read(i)
    end
    i = Colours.remove_trailing_end_from(i)
    if i.is_a? Array
      i = i.join(' ').strip
    end
    if name_of_the_colour
      if name_of_the_colour.respond_to?(:include?) and
         name_of_the_colour.include?('/')
        name_of_the_colour = File.basename(name_of_the_colour)
      end
    end
    name_of_the_colour = name_of_the_colour.to_sym
    return ::Colours::SupportForHTMLColours.send(name_of_the_colour, i)
  end

  # ========================================================================= #
  # === Colours.sanitize_line
  #
  # This method will replace one line with the proper R,G,B valid entries.
  # ========================================================================= #
  def self.sanitize_line(entry)
    all_potential_matches = entry.scan(/<(\w+)>/).flatten
    all_potential_matches.each {|substring|
      entry.gsub!(/<\/>/, rev)
      entry.gsub!(/<#{substring}>/, rgb_format(substring))
    }
    begin
      require 'roebe/modules/remove_html.rb'
    rescue LoadError; end
    if Object.const_defined?(:Roebe) and
       Roebe.respond_to?(:remove_html)
      entry = Roebe.remove_html[entry]
    end
    return entry
  end

  # ========================================================================= #
  # === Colours.read_and_display_this_file
  #
  # This method accepts a file path - in other words, the input should be
  # the location of a specific file on your filesystem - and then simply
  # reads in the content of said file via File.readlines().
  #
  # Content such as <slateblue> found in this file will be replaced
  # with the respective R,G,B substring value.
  #
  # In the test/ subdirectory there is an example file for this - have
  # a look there for more information.
  # ========================================================================= #
  def self.read_and_display_this_file(
      i = TEST_FILE
    )
    i = i.to_s
    if File.exist? i
      new_array = []
      data = File.readlines(i)
      data.each {|entry|
        # ======================================================================= #
        # Check whether the entry has a <> tag:
        # ======================================================================= #
        if entry.include?('<') and entry.include?('>') and 
           entry.include?('</>')
          # ================================================================= #
          # Ok, we may assume that something like <slateblue> is there.
          # ================================================================= #
          entry = sanitize_line(entry)
        end
        new_array << entry
      }
      e new_array # For now we will simply output that modified Array.
    else
      e "Notice: The file at `#{sfile(i)}` does not exist."
    end
  end; self.instance_eval { alias read_file read_and_display_this_file } # === Colours.read_file

  # ========================================================================= #
  # === Colours.does_this_line_include_a_html_colour?
  #
  # This method can be used to determine whether the given input-string
  # contains a valid HTML colour or whether it does not.
  #
  # Returns: a boolean.
  #
  # Invocation examples:
  #
  #   Colours.does_this_line_include_a_html_colour? "<green>yo there</green> <orange>getline() function</orange>" # => true
  #   Colours.does_this_line_include_a_html_colour? "foo bar" # => false
  #
  # ========================================================================= #
  def self.does_this_line_include_a_html_colour?(
      line
    )
    html_colours?.any? {|entry|
      line.include? entry
    }
  end

  # ========================================================================= #
  # === Colours.convert_hex_to_rgb
  #
  # This method will convert e. g. #baf185 to [186, 241, 133]. Thus it
  # will return an Array, denoting the R, G, B values.
  #
  # How to do this conversion on your own?
  #
  #   (1) Get the 2 left digits of the hex color code and convert
  #       to decimal value to get the red color level.
  #   (2) Get the 2 middle digits of the hex color code and
  #       convert to decimal value to get the green color level.
  #   (3) Get the 2 right digits of the hex color code and
  #       convert to decimal value to get the blue color level.
  #
  # Usage example:
  #
  #   Colours.convert_hex_to_rgb('#baf185') # => [186, 241, 133]
  #
  # ========================================================================= #
  def self.convert_hex_to_rgb(hex)
    if hex.is_a? Array
      hex = hex.first
    end
    hex = hex.to_s.dup
    hex.delete!('#') if hex.include? '#'
    array = [] # We will return this Array.
    r = hex[0,2].to_i(16)
    g = hex[2,2].to_i(16)
    b = hex[4,2].to_i(16)
    array << r << g << b
    return array
  end; self.instance_eval { alias hex_to_rgb convert_hex_to_rgb } # === Colours.hex_to_rgb

  # ========================================================================= #
  # === Colours.convert_hex_code_to_RGBA_array
  #
  # RGBA stands for "red, green, blue, alpha". Alpha indicates how
  # opaque each pixel is.
  #
  # The usual values for the alpha parameter, aka the last parameter,
  # is a number between 0.0 (which means "fully transparent") and
  # the number 1.0 (which means "not transparent at all").
  #
  # Note that this method is similar to Colours.convert_hex_to_rgb(hex),
  # but it has a fourth argument, aka A (for Alpha), on top of
  # the RGB values.
  #
  # Usage example:
  #
  #   Colours.convert_hex_code_to_RGBA_array('#baf185') # => [186, 241, 133]
  #
  # ========================================================================= #
  def self.convert_hex_code_to_RGBA_array(
      i, default_alpha_value = 1.0
    )
    rgba_array = convert_hex_to_rgb(i)
    rgba_array << default_alpha_value
    return rgba_array
  end

  # ========================================================================= #
  # === Colours.reset_the_line
  #
  # This method will reset the current line. This can then be used in a
  # progress indicator application.
  # ========================================================================= #
  def self.reset_the_line
    STDOUT.write("\u001b[1000D") # Move to the left by 1000 characters. Aka reset.
    STDOUT.flush # This line will force the output to appear immediately, 
  end

end

if __FILE__ == $PROGRAM_NAME
  # ========================================================================= #
  # Example code demonstrating the method defined above:
  # ========================================================================= #
  index = 1
  loop {
    Colours.reset_the_line
    print 'OK! '+
           Random.rand(11).to_s 
    index += 1
    break if index == 50
    sleep 0.3
  }
  puts
end