#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'colours/toplevel_methods/use_colours.rb'
# =========================================================================== #
module Colours

  require 'colours/toplevel_methods/set_last_colour_used.rb'

  # ========================================================================= #
  # === @use_colours
  #
  # Whether we will use colours or whether we will not. This can be
  # used to globally disable colours in every project using the
  # Colours gem.
  # ========================================================================= #
  @use_colours = true

  # ========================================================================= #
  # === @use_html_colours
  #
  # This is a separate variable so that we can selectively disable or
  # enable the colours for the Konsole submodule. Some terminals do
  # not respond nicely to the R,G,B values that the Konsole submodule
  # may require.
  # ========================================================================= #
  @use_html_colours = true

  # ========================================================================= #
  # === Colours.use_colours?
  #
  # Use this method to find out whether we wish to use colours or
  # whether we do not.
  # ========================================================================= #
  def self.use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === Colours.enable_html_colours
  #
  # Use this method to specifically enable konsole-colours.
  # ========================================================================= #
  def self.enable_html_colours
    @use_html_colours = true
  end; self.instance_eval { alias enable_konsole_colours enable_html_colours } # === Colours.enable_konsole_colours

  # ========================================================================= #
  # === Colours.use_html_colours?
  #
  # Query whether we will use the HTML colours or whether we will not.
  # ========================================================================= #
  def self.use_html_colours?
    @use_html_colours
  end; self.instance_eval { alias use_konsole?         use_html_colours? } # === Colours.use_konsole?
       self.instance_eval { alias use_konsole_colours? use_html_colours? } # === Colours.use_konsole_colours??

  # ========================================================================= #
  # === Colours.disable_html_colours
  #
  # Disable the HTML colours.
  # ========================================================================= #
  def self.disable_html_colours
    @use_html_colours = false
  end

  # ========================================================================= #
  # === Colours.disable_colours
  #
  # This method can be used to disable the colours on the Colours
  # namespace.
  # ========================================================================= #
  def self.disable_colours(
      be_verbose = false
    )
    puts 'Disabling colours next.' if be_verbose
    @use_colours = false
  end; self.instance_eval { alias disable disable_colours } # === Colours.disable

  # ========================================================================= #
  # === Colours.enable_colours
  #
  # Use this method to enable colours for the whole Colours namespace.
  # ========================================================================= #
  def self.enable_colours(
      be_verbose = false
    )
    puts 'Enabling colours next.' if be_verbose
    @use_colours = true # Defined below.
  end; self.instance_eval { alias enable enable_colours } # === Colours.enable

  # ========================================================================= #
  # === Colours.use_colours=
  #
  # Determine whether we will use colours or whether we will not.
  #
  # The first input argument to this method should be a Bool, such
  # as true or false.
  # ========================================================================= #
  def self.use_colours=(
      new_value = true
    )
    @use_colours = new_value
  end; self.instance_eval { alias set_use_colours use_colours= } # === Colours.set_use_colours

  # ========================================================================= #
  # === use_colours?
  #
  # Delegate towards the class-method here. Unsure whether this method
  # is really needed, but let's keep it for now. After all we can
  # use "include Colours" to add that method.
  # ========================================================================= #
  def use_colours?
    ::Colours.use_colours?
  end

  # ========================================================================= #
  # === Colours.report_whether_colours_will_be_used
  # ========================================================================= #
  def self.report_whether_colours_will_be_used
    if @use_colours
      puts 'Yes, colours will be used.'
    else
      puts 'Colours are currently disabled.'
    end
  end

# =========================================================================== #
# This file includes a standalone Colour class, as part of the
# Colours namespace.
# =========================================================================== #
class Colour # === Colours::Colour

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = nil
    )
    reset
    set_input(optional_input) if optional_input
  end

  # ========================================================================= #
  # === set_r
  # ========================================================================= #
  def set_r(i = nil)
    @r = i
  end

  # ========================================================================= #
  # === set_g
  # ========================================================================= #
  def set_g(i = nil)
    @g = i
  end

  # ========================================================================= #
  # === set_b
  # ========================================================================= #
  def set_b(i = nil)
    @b = i
  end

  # ========================================================================= #
  # === random_colour?
  # ========================================================================= #
  def random_colour?(
      optional_text = nil
    )
    if optional_text
      write_this_in_random_colour(optional_text)
    else
      ::Colours.random_colour?
    end
  end

  # ========================================================================= #
  # === set_rgb
  # ========================================================================= #
  def set_rgb(r,g,b)
    set_r(r)
    set_g(g)
    set_b(b)
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = nil)
    @input = i
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    set_r
    set_g
    set_b
  end

  # ========================================================================= #
  # === r?
  # ========================================================================= #
  def r?
    @r
  end

  # ========================================================================= #
  # === g?
  # ========================================================================= #
  def g?
    @g
  end

  # ========================================================================= #
  # === b?
  # ========================================================================= #
  def b?
    @b
  end

  # ========================================================================= #
  # === output_this_string
  # ========================================================================= #
  def output_this_string(
      i = DEFAULT_TEXT+N,
      r = r?,
      g = g?,
      b = b?
    )
    result = return_this_string(i,r,g,b)
    e result
  end; alias output_this                 output_this_string # === output_this
       alias output                      output_this_string # === output
       alias write_this_in_random_colour output_this_string # === write_this_in_random_colour

  # ========================================================================= #
  # === return_this_string
  # ========================================================================= #
  def return_this_string(
      i = DEFAULT_TEXT+N,
      r = r?,
      g = g?,
      b = b?
    )
    i = DEFAULT_TEXT if i == :default
    i = i.join(' ') if i.is_a? Array
    i = i.to_s.dup
    i = build_this_rgb_string(r, g, b)+i
    return i
  end

  # ========================================================================= #
  # === build_this_rgb_string
  # ========================================================================= #
  def build_this_rgb_string(r, g, b)
    Colours.build_this_rgb_string(r,g,b)
  end

  # ========================================================================= #
  # === random_colour
  #
  # Return a random colour here.
  # ========================================================================= #
  def random_colour(this_string_to_display = 'test')
    return_this_string(
      this_string_to_display, random_value, random_value, random_value
    )
  end; alias return_random_colour random_colour # === return_random_colour

  # ========================================================================= #
  # === seed_rgb_values
  #
  # Use this method to randomly obtain R,G,B values.
  # ========================================================================= #
  def seed_rgb_values
    set_rgb(random_value, random_value, random_value)
  end

  # ========================================================================= #
  # === random_value
  # ========================================================================= #
  def random_value
    rand(255)
  end

end; end